/*
* matrix.h
*
* $Id: matrix.h,v 1.3 2003/02/18 23:23:53 Paul Exp $
*
* Copyright (C) 1998-2003 by Paul Thiessen, Chemical Graphics
*                 paul@chemicalgraphics.com
*           http://www.chemicalgraphics.com/paul
*
* Defines a 4x4 matrix type used for 3-d transformations, and a vector
* type used to hold and manipulate 3-d coordinates.
*/

#ifndef __vrml2pov_matrix_h_
#define __vrml2pov_matrix_h_

#include <math.h>
#include <iostream>

#include "global.h"
#include "fieldtyp.h"

class Matrix {
public:
	F_TYPE m[16];
	Matrix& operator=(const Matrix& o) {
		for (int i=0; i<16; i++) m[i]=o.m[i];
		return *this;
	}
	Matrix(F_TYPE m0 =1, F_TYPE m1 =0, F_TYPE m2 =0, F_TYPE m3 =0,
		F_TYPE m4 =0, F_TYPE m5 =1, F_TYPE m6 =0, F_TYPE m7 =0,
		F_TYPE m8 =0, F_TYPE m9 =0, F_TYPE m10 =1, F_TYPE m11 =0,
		F_TYPE m12 =0, F_TYPE m13 =0, F_TYPE m14 =0, F_TYPE m15 =1) {
		m[0]=m0;  m[1]=m1;	m[2]=m2; m[3]=m3;
		m[4]=m4;  m[5]=m5;	m[6]=m6;  m[7]=m7;
		m[8]=m8;  m[9]=m9;	m[10]=m10; m[11]=m11;
		m[12]=m12; m[13]=m13; m[14]=m14; m[15]=m15;
	}
};

//
// This function is *not* used to output the matrix verbatim, but to
// create a matrix <> transformation object as POV understands it.
//
std::ostream& operator<<(std::ostream&, Matrix*); // implemented in povout.cc

inline void set_translation_matrix(Matrix* m, SFVec3f* v, int n =1)
{
	m->m[0]=1; m->m[1]=0; m->m[2]=0; m->m[3]=n*(v->value[0]);
	m->m[4]=0; m->m[5]=1; m->m[6]=0; m->m[7]=n*(v->value[1]);
	m->m[8]=0; m->m[9]=0; m->m[10]=1; m->m[11]=n*(v->value[2]);
	m->m[12]=0; m->m[13]=0; m->m[14]=0; m->m[15]=1;
	return;
}

inline void set_scale_matrix(Matrix* m, SFVec3f* v)
{
	m->m[0]=v->value[0]; m->m[1]=0; m->m[2]=0; m->m[3]=0;
	m->m[4]=0; m->m[5]=v->value[1]; m->m[6]=0; m->m[7]=0;
	m->m[8]=0; m->m[9]=0; m->m[10]=v->value[2]; m->m[11]=0;
	m->m[12]=0; m->m[13]=0; m->m[14]=0; m->m[15]=1;
	return;
}

class Vector {
public:
	F_TYPE v[3];
	Vector& operator=(const Vector& o) {
		v[0]=o.v[0];
		v[1]=o.v[1];
		v[2]=o.v[2];
		return *this;
	}
	Vector(const SFColor* o) { // make Vector from SFColor
		v[0]=o->value[0];
		v[1]=o->value[1];
		v[2]=o->value[2];
	}
	Vector(F_TYPE x =0, F_TYPE y =0, F_TYPE z =0) {
		v[0]=x;
		v[1]=y;
		v[2]=z;
	}
};

inline std::ostream& operator<<(std::ostream& s, Vector v) {
	return s << '<' << v.v[0] << ',' << v.v[1] << ',' << v.v[2] << '>';
}

inline void set_rotation_matrix(Matrix* m, SFRotation* r, int n =1)
{
	Vector u;
	double length, c, s, t;
	length=sqrt((r->value[0])*(r->value[0])+
		(r->value[1])*(r->value[1])+
		(r->value[2])*(r->value[2]));
	u.v[0]=r->value[0]/length;
	u.v[1]=r->value[1]/length;
	u.v[2]=r->value[2]/length;
	c=cos(n*(r->value[3]));
	s=sin(n*(r->value[3]));
	t=1.0-c;
	
	m->m[0]=u.v[0]*u.v[0]*t+c;
	m->m[1]=u.v[0]*u.v[1]*t-u.v[2]*s;
	m->m[2]=u.v[0]*u.v[2]*t+u.v[1]*s;
	m->m[3]=0;
	
	m->m[4]=u.v[0]*u.v[1]*t+u.v[2]*s;
	m->m[5]=u.v[1]*u.v[1]*t+c;
	m->m[6]=u.v[1]*u.v[2]*t-u.v[0]*s;
	m->m[7]=0;
	
	m->m[8]=u.v[0]*u.v[2]*t-u.v[1]*s;
	m->m[9]=u.v[1]*u.v[2]*t+u.v[0]*s;
	m->m[10]=u.v[2]*u.v[2]*t+c;
	m->m[11]=0;
	
	m->m[12]=m->m[13]=m->m[14]=0; m->m[15]=1;
	
	return;
}

inline void apply_transformation(Matrix* m, Vector* v)
{
	Vector t;
	
	t.v[0]=m->m[0]*v->v[0]+m->m[1]*v->v[1]+m->m[2]*v->v[2]+m->m[3];
	t.v[1]=m->m[4]*v->v[0]+m->m[5]*v->v[1]+m->m[6]*v->v[2]+m->m[7];
	t.v[2]=m->m[8]*v->v[0]+m->m[9]*v->v[1]+m->m[10]*v->v[2]+m->m[11];
	v->v[0]=t.v[0];
	v->v[1]=t.v[1];
	v->v[2]=t.v[2];
	return;
}

inline void apply_transformation(Matrix* m, F_TYPE* v)
{
	Vector t;
	
	t.v[0]=m->m[0]*v[0]+m->m[1]*v[1]+m->m[2]*v[2]+m->m[3];
	t.v[1]=m->m[4]*v[0]+m->m[5]*v[1]+m->m[6]*v[2]+m->m[7];
	t.v[2]=m->m[8]*v[0]+m->m[9]*v[1]+m->m[10]*v[2]+m->m[11];
	v[0]=t.v[0];
	v[1]=t.v[1];
	v[2]=t.v[2];
	return;
}

inline void compose_into(const Matrix* A, const Matrix* B, Matrix* C)
{
	for (int i=0; i<4; i++) {
		for (int j=0; j<4; j++) {
			C->m[4*i+j]=0;
			for (int s=0; s<4; s++) {
				C->m[4*i+j] += (A->m[4*i+s])*(B->m[4*s+j]);
			}
		}
	}
	return;
}

//
// The following inverts a 4x4 matrix whose bottom row is { 0 0 0 1 } - i.e.
// a matrix composed only of rotate, scale, and translate matrixes.
//

inline void invert_into(Matrix* A, Matrix* I)
{
	F_TYPE& a=A->m[0], b=A->m[1], c=A->m[2], d=A->m[3],
		e=A->m[4], f=A->m[5], g=A->m[6], h=A->m[7],
		i=A->m[8], j=A->m[9], k=A->m[10], l=A->m[11];
	F_TYPE denom=(-(c*f*i) + b*g*i + c*e*j - a*g*j - b*e*k + a*f*k);
	I->m[0]= (-(g*j) + f*k)/denom;
	I->m[1]= (c*j - b*k)/denom;
	I->m[2]= (-(c*f) + b*g)/denom;
	I->m[3]= (d*g*j - c*h*j - d*f*k + b*h*k + c*f*l - b*g*l)/denom;
	I->m[4]= (g*i - e*k)/denom;
	I->m[5]= (-(c*i) + a*k)/denom;
	I->m[6]= (c*e - a*g)/denom;
	I->m[7]= (-(d*g*i) + c*h*i + d*e*k - a*h*k - c*e*l + a*g*l)/denom;
	I->m[8]= (-(f*i) + e*j)/denom;
	I->m[9]= (b*i - a*j)/denom;
	I->m[10]= (-(b*e) + a*f)/denom;
	I->m[11]= (d*f*i - b*h*i - d*e*j + a*h*j + b*e*l - a*f*l)/denom;
	I->m[12]=0; I->m[13]=0; I->m[14]=0; I->m[15]=1;
	return;
}

#endif // __vrml2pov_matrix_h_
