/*
* node.h
*
* $Id: node.h,v 1.5 2003/02/18 23:23:53 Paul Exp $
*
* Copyright (C) 1998-2003 by Paul Thiessen, Chemical Graphics
*                 paul@chemicalgraphics.com
*            http://www.chemicalgraphics.com/paul
*
* Defines the node and related classes. Note that for a nodeBodyElement that
* is a "fieldId fieldValue" type, the fieldValue pointer passed to the
* constructor remains as the element's value - a new fieldType object is
* *not* created, thus the pointer passed should be for a fieldType oject
* created by "new".
*/

#ifndef __vrml2pov_node_h_
#define __vrml2pov_node_h_

#include <string.h>

#include "global.h"

class fieldType;
class SFNode;

class statement {};

class nodeVariable {
public:
	char* fieldId;
	fieldType* fieldValue;
	nodeVariable(const char* id, fieldType* p) {
		if (id) {
			fieldId=new char[strlen(id)+1];
			if (!fieldId) {
				std::cerr << "Ran out of memory\n";
                abort();
			}
			strcpy(fieldId,id);
		} else
			fieldId=NULL;
		fieldValue=p;
	}
	~nodeVariable() {
		if (fieldId) delete fieldId;
	}
};

class nodeISarg {
public:
	char* fieldId;
	char* argId;
	nodeISarg(const char* id, const char* arg) {
		if (id) {
			fieldId=new char[strlen(id)+1];
			if (!fieldId) {
				std::cerr << "Ran out of memory\n";
                abort();
			}
			strcpy(fieldId,id);
		} else
			fieldId=NULL;
		if (arg) {
			argId=new char[strlen(arg)+1];
			if (!argId) {
				std::cerr << "Ran out of memory\n";
                abort();
			}
			strcpy(argId,arg);
		} else
			argId=NULL;
	}
	~nodeISarg() {
		if (fieldId) delete fieldId;
		if (argId) delete argId;
	}
};

class nodeStatement : public statement {
public:
	char* nodeNameId;
	SFNode* node;
	nodeStatement(char* name =NULL, SFNode* n =NULL) {
		if (name) {
			nodeNameId=new char[strlen(name)+1];
			if (!nodeNameId) {
				std::cerr << "Ran out of memory\n";
                abort();
			}
			strcpy(nodeNameId,name);
		} else
			nodeNameId=NULL;
		node=n;
	}
	~nodeStatement() {
		if (nodeNameId) delete nodeNameId;
	}
};

class ParserObject; // defined in parser.h

fieldType* parse_SFNode(ParserObject*);
fieldType* parse_MFNode(ParserObject*);
int skip_node(ParserObject*);

#endif // __vrml2pov_node_h_
