/*
* povout.h
*
* $Id: povout.h,v 1.4 2003/02/18 23:23:53 Paul Exp $
*
* Copyright (C) 1998-2003 by Paul Thiessen, Chemical Graphics
*                 paul@chemicalgraphics.com
*           http://www.chemicalgraphics.com/paul
*
* Header for all modules that do some sort of POV output. Prototypes for
* all POV output routines should be in here, even if they're implemented
* in a module other than povout.cc.
*/

#ifndef __vrml2pov_povout_h_
#define __vrml2pov_povout_h_

#include <iomanip>

#include "node.h"
#include "matrix.h"

#ifndef __vrml2pov_povout_cc_
extern Matrix* current_transformation;
extern DLList<Matrix> transformation_Stack;
extern int indent, ii;
extern int use_transparency;
extern F_TYPE transparency;
extern char finish[];
extern SFColor* emissiveColor;
#endif

inline void push_transformation(Matrix* m)
{
	transformation_Stack.add(current_transformation);
	current_transformation=m;
}
inline void pop_transformation(void)
{
	delete current_transformation;
	current_transformation=transformation_Stack.current();
    transformation_Stack.remove();
}

#define INDENT setw(indent) << ""

void do_Appearance(SFNode*);
void do_Box(SFNode*);
void do_Collision(SFNode*);
void do_Cone(SFNode*);
void do_Cylinder(SFNode*);
void do_Extrusion(SFNode*);
void do_Group(SFNode*);
void do_IndexedFaceSet(SFNode*);
void do_IndexedLineSet(SFNode*);
void do_Inline(SFNode*);
void do_Material(SFNode*);
void do_PointSet(SFNode*);
void do_Shape(SFNode*);
void do_Sphere(SFNode*);
void do_Transform(SFNode*);
void do_Viewpoint(SFNode*);

#endif // __vrml2pov_povout_h_
