/*
* proto.cpp
*
* $Id: proto.cpp,v 1.4 2003/02/18 23:23:53 Paul Exp $
*
* Copyright (C) 1998-2003 by Paul Thiessen, Chemical Graphics
*                 paul@chemicalgraphics.com
*            http://www.chemicalgraphics.com/paul
*
* Implemements the PROTO parser. Returns a pointer to a new PROTO
* object if successful, NULL if not.
*/

#include <stdio.h>
#include <ctype.h>

#include "parser.h"
#include "proto.h"

using namespace std;

protoStatement* parse_PROTO(ParserObject* p)
{
	ReadFileObject& r=*(p->current_file); // for convenience
	
	// sanity check
	
	//  if (strcmp(r.current_word,"PROTO")!=0) {
	//   cerr << "parse_PROTO() called at inappropriate time...\n";
	//    return NULL;
	//  }
	
	if (!r.get_next_word())
		return (protoStatement*)r.parse_error("nodeTypeId",NULL);
	protoStatement* proto=new protoStatement;
	if (!proto) OUTMEM;
	
	// get the PROTO's name
	
    if (!is_valid_id(r.current_word))
		return (protoStatement*)r.parse_error("valid nodeTypeId",r.current_word);
	proto->nodeTypeId=new char[strlen(r.current_word)+1];
	if (!proto->nodeTypeId) OUTMEM;
	strcpy(proto->nodeTypeId,r.current_word);
	if (p->isVerbose) cout << proto->nodeTypeId << '\n';
	
	// get [
	
	if (!r.get_next_word())
		return (protoStatement*)r.parse_error("[",NULL);
	if (strcmp(r.current_word,"[")!=0)
		return (protoStatement*)r.parse_error("[",r.current_word);
	p->in_PROTO_declarations=TRUE;
	
	// get interface declarations until ]
	
	if (!r.get_next_word())
		return (protoStatement*)r.parse_error("interface declarations",NULL);
	while (strcmp(r.current_word,"]")!=0) {
		
		//
		// Store only names of event[In,Out]s, to identify (and ignore) later...
		// This is admittedly a bit of a hack, since we don't really care about the
		// the events, except to store their names so that if they're invoked in
		// a nodeBodyElement we can know to ignore them.
		//
		
		if (strcmp(r.current_word,"eventIn")==0 ||
			strcmp(r.current_word,"eventOut")==0) {
			if (!r.get_next_word() || !r.get_next_word())
				return (protoStatement*)r.parse_error("event[In,Out] declaration",NULL);
            if (!is_valid_id(r.current_word))
				return (protoStatement*)r.parse_error("event[In,Out]Id",r.current_word);
			interfaceDeclaration* decl=new interfaceDeclaration(r.current_word);
			if (!decl) OUTMEM;
			decl->defaultValue=new Event();
			if (!decl->defaultValue) OUTMEM;
			proto->interfaceDeclarations.add(decl);
		}
		
		// read and store field & exposedField type, id, and default value
		
		else if (strcmp(r.current_word,"field")==0 ||
			strcmp(r.current_word,"exposedField")==0) {
			if (!r.get_next_word())
				return (protoStatement*)r.parse_error("fieldType",NULL);
			unsigned char type=get_type(r.current_word);
			if (!type)
				return (protoStatement*)r.parse_error("fieldType",r.current_word);
			if (!r.get_next_word())
				return (protoStatement*)r.parse_error("fieldId",NULL);
            if (!is_valid_id(r.current_word))
				return (protoStatement*)r.parse_error("valid fieldId",r.current_word);
			interfaceDeclaration* decl=new interfaceDeclaration(r.current_word);
			if (!decl) OUTMEM;
			if (!r.get_next_word())
				return (protoStatement*)r.parse_error(p->type_name[type],NULL);
			if (type==SFNODE && strcmp(r.current_word,"NULL")==0)
				decl->defaultValue=NULL;
			else {
				decl->defaultValue=(*p->parse_type[type])(p);
				if (!decl->defaultValue) {
					cerr << "Error parsing type " << p->type_name[type] << '\n';
					return NULL;
				}
			}
			proto->interfaceDeclarations.add(decl);
			
		} else
			return (protoStatement*)r.parse_error(
			"[ eventIn | field | exposedField | eventOut ]",r.current_word);
		
		if (!r.get_next_word())
			return (protoStatement*)r.parse_error("interface declarations or ]",NULL);
	} // while declarations are to be read
	p->in_PROTO_declarations=FALSE;
	
	//
	// When inside a PROTO body, the scope of DEF, USE, PROTO, and IS statements
	// must be limited to the inside of the PROTO. Thus, whenever a PROTO is
	// being read, it must have its own context for these things. The push_ and
	// pop_context functions set this up in a manner that also makes the context
	// correct for nested PROTOs.
	//
	
	p->push_context(proto);
	
	// get protoBody between { and }
	
	if (!r.get_next_word())
		return (protoStatement*)r.parse_error("{",NULL);
	if (strcmp(r.current_word,"{")!=0)
		return (protoStatement*)r.parse_error("{",r.current_word);
	
	if (!r.get_next_word())
		return (protoStatement*)r.parse_error("protoBody statement or }",NULL);
	
	while (strcmp(r.current_word,"}")!=0) {
		
		// skip EXTERNPROTO with warning (eventually)
		
		if (strcmp(r.current_word,"EXTERNPROTO")==0) {
			cerr << r.current_word << " currently unimplemented... dang.\n";
			return NULL;
		}
		
		// skip ROUTE
		
		if (strcmp(r.current_word,"ROUTE")==0) {
			if (!skip_route(p)) {
				cerr << "Error in ROUTE statement\n";
				return NULL;
			}
		}
		
		// deal with nodes or nested PROTOs
		
		if (strcmp(r.current_word,"PROTO")==0) {
			if (p->isVerbose) cout << "Parsing nested PROTO\n";
			proto->local_PROTOs.add(parse_PROTO(p));
			if (!(proto->local_PROTOs.current())) {
				cerr << "Error in nested PROTO statement\n";
				return NULL;
			}
		} else {
			proto->local_Nodes.add((SFNode*)parse_SFNode(p));
			if (!(proto->local_Nodes.current())) {
				cerr << "Error in PROTO body node statement\n";
				return NULL;
			}
		}
		
		if (!r.get_next_word())
			return (protoStatement*)r.parse_error("protoBody statement or }",NULL);
		
	} // while protoBody being read
	
	// Return to the previous context.
	
	p->pop_context();
	
	// Check for unused arguments in this PROTO - causes memory wastage...
	
	for (int i=0; i<(proto->interfaceDeclarations.n_items()); i++) {
		if (p->isVerbose && !(proto->interfaceDeclarations.current()->used))
			cout << "Warning: unused argument " <<
			proto->interfaceDeclarations.current()->fieldId <<
			" in PROTO " << proto->nodeTypeId << '\n';
		proto->interfaceDeclarations.next();
	}
	return proto;
}
