/*
* proto.h
*
* $Id: proto.h,v 1.4 2003/02/18 23:23:54 Paul Exp $
*
* Copyright (C) 1998-2003 by Paul Thiessen, Chemical Graphics
*                 paul@chemicalgraphics.com
*            http://www.chemicalgraphics.com/paul
*
* Defines the PROTO and related classes.
*
* Like fields in nodes, the fieldValue passed to the interfaceDeclaration
* constructor will be stored as a pointer, not copied, so that value must
* have been created by "new".
*/

#ifndef __vrml2pov_proto_h_
#define __vrml2pov_proto_h_

#include <string.h>

#include "global.h"
#include "fieldtyp.h"
#include "list.h"
#include "node.h"

//
// For now, only field statements of interfaceDeclaration fields will be
// defined, and "field" and "exposedField" are treated identically.
//
class fieldType;

class interfaceDeclaration {
public:
	char* fieldId;
	fieldType* defaultValue;
	unsigned char used;
	interfaceDeclaration(const char* id) {
		if (id) {
			fieldId=new char[strlen(id)+1];
			if (!fieldId) {
				std::cerr << "Ran out of memory\n";
                abort();
			}
			strcpy(fieldId,id);
		} else
			fieldId=NULL;
		defaultValue=NULL;
		used=FALSE;
	}
	~interfaceDeclaration() {
		if (fieldId) delete fieldId;
	}
};

class protoStatement : public statement {
public:
	char* nodeTypeId;
	DLList<interfaceDeclaration> interfaceDeclarations;
	DLList<protoStatement> local_PROTOs;
	DLList<nodeStatement> local_DEFs;
	DLList<SFNode> local_Nodes;
	protoStatement() {
		nodeTypeId=NULL;
	}
	~protoStatement() {
		if (nodeTypeId) delete nodeTypeId;
        interfaceDeclarations.empty_and_delete_all();
        local_PROTOs.empty_and_delete_all();
        local_DEFs.empty_and_delete_all();
	}
};

protoStatement* parse_PROTO(ParserObject*);

#endif // __vrml2pov_proto_h_
